<?php
namespace PME\Controllers\Relatorios;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use DateTime;

class EmissaoCardSemanaController extends \Zeedhi\Framework\Controller\Report {

  protected $birtStrategy;
  protected $utilities;

  public function __construct(ReportService $reportService, $birtStrategy, $utilities) {
    $this->birtStrategy  = $birtStrategy;
    $this->utilities     = $utilities;

    parent::__construct($reportService);
  }

  protected $reportMapping = array (
    'PME01000' => 
    array (
      'strategy' => 'BIRT',
      'parameterMapping' => 
      array (
        "IMPCOMENSAIS" => "P_IMPCOMENS",
        "CDFILIAL"     => "P_CDFILIAL",
        "NMFILIAGRE"   => "P_NMFILIAGRE",
        "QUEBRA"       => "P_QUEBRA",
        "TIPOCARD"     => "P_CDTIPOCARD",
        "CDFILCENTPME" => "P_CDFILCENTPME",
        "DTDE"         => "P_DTINICIAL",
        "DTATE"        => "P_DTFINAL",
        "SERVICO"      => "P_CDSERVFILI",
        "NRORG"        => "P_NRORG",
        "IMPMSG"       => "P_IMPMSG",
        "ORDENA"       => "P_ORDENA",
        "DIASCONSUMO"  => "P_DIASCONSUMO"
      )
    )
  );
  
  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
    $row = $request->getRow();
    $nrorg = $this->utilities->getNrOrg();
    
    if (empty($row['DIASCONSUMO'])) $row['DIASCONSUMO'] = '1_2_3_4_5_6_7';
    else $row['DIASCONSUMO'] = implode("_", $row['DIASCONSUMO']);

    $row['DTDE'] = $this->firstDayOfWeek($row);
    $row['DTATE'] = $this->lastDayOfWeek($row);

    if(empty($row['SERVICO'])) $row['SERVICO'] = 'T';
    else if(is_array($row['SERVICO'])) $row['SERVICO'] = join("_", $row['SERVICO']);
    
    if(empty($row['UNIDADE'])) $row['NMUNIDADE'] = null;
    else $row['NMUNIDADE'] = $row['UNIDADE']['CDFILIAL'].' '.$row['UNIDADE']['NMFILIAL'];

    $row['QUEBRA'] = empty($row['QUEBRA']) ? 'N' : 'S';

    if(empty($row['IMPMSG'])) $row['IMPMSG'] = 'N';
    else {
      $row['QUEBRA'] = 'S';
      $row['IMPMSG'] = 'S';
    }
    
    $row['NRORG'] = $nrorg;
    
    $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
    $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
  }
  
  public function firstDayOfWeek($row){
    $date = str_replace("/", "-", $row['DTDE']);
    $nbDay = date('N', strtotime($date));
    $monday = new DateTime($date);
    $monday->modify('-'.($nbDay-1).' days');
    $dtDe= $monday->format('d/m/Y');
    return $dtDe;
  }
  
  public function lastDayOfWeek($row){
    $date = str_replace("/", "-", $row['DTATE']);
    $nbDay = date('N', strtotime($date));
    $sunday = new DateTime($date);
    $sunday->modify('+'.(7-$nbDay).' days');
    $dtAte= $sunday->format('d/m/Y');
    return $dtAte;
  }

}