import {
  Http, Loader, Metadata, I18n, Datasource, IEventParam, IDictionary,
} from '@zeedhi/core';

import {
  Form, Button, DialogService, GridEditable, Grid, Card, Modal, ModalService,
} from '@zeedhi/common';
  
export class Ger96000CentralController {
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private agregadaIsVisible = false;
  
  public selectedCentral = '';
  
  private modalRelatorioCentral!: Modal;
  
  public reportController = Loader.getInstance('ReportController');

  public save({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formCentral');
    const grid: GridEditable = Metadata.getInstance('CENTRAL');
    const editedRows = grid.getEditedRows();
    
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (!row.__is_new) {
          row.__is_new = false;
        }
      });
      
      if (grid) grid.datasource.loading = true;
      Http.post('/salvaCentral', { editedRows }).then((response: IDictionary) => {
        let param;
        if (response.data.length === 0) {
          grid.datasource.get();
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          this.gridEditableController.cancelEditing({ component });
        } else {
          const mensagem = response.data.dataset.mensagem[0];
          
          if (mensagem.indexOf('ERROR_E_AGREGADA') === 0) {
            param = mensagem.split(':');
            this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_E_AGREGADA', { PARAM1: param[1] }), 'error');
          } else if (mensagem.indexOf('ERROR_JA_CADASTRADA') === 0) {
            param = mensagem.split(':');
            this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_UNID_CENTRAL', { PARAM1: param[1] }), 'error');
          } else if (mensagem.indexOf('ERROR_CENTRAL_M') === 0) {
            this.utilities.showAlert('alertError', 'pme::ERROR_CENTRAL_M', 'error');
            // this.gridEditableController.cancelEditing({ component });
          } else {
            this.utilities.showAlert('alertError', mensagem, 'error');
          }
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      }).finally(() => {
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  public cancelEditingTelaCentral({ component }: IDictionary) {
    this.gridEditableController.cancelEditing({ component });
  }
  
  private cardAgregadaVisible() {
    const card = Metadata.getInstance<Card>('agregadaCard');
    const gridCentral: GridEditable = Metadata.getInstance('CENTRAL');
    try {
      if (!gridCentral.editing) {
        card.isVisible = true;
        this.agregadaIsVisible = true;
      } else {
        card.isVisible = false;
        this.agregadaIsVisible = false;
      }
    } catch (Error) {
      card.isVisible = false;
      this.agregadaIsVisible = false;
    }
  }
  
  public closeCardAgregada() {
    const card = Metadata.getInstance<Card>('agregadaCard');
    const gridAgregada: GridEditable = Metadata.getInstance('AGREGADA');
    this.gridEditableController.cancelEditing({ component: gridAgregada });
    
    card.isVisible = false;
    this.agregadaIsVisible = false;
  }
  
  get gridColsReturnValue(): number {
    return this.agregadaIsVisible ? 6 : 12;
  }

  public carregaCentrais() {
    const central: GridEditable = Metadata.getInstance('CENTRAL');
    central.datasource.get().then(() => {
      central.datasource.currentRow = central.datasource.data[0];
    });
  }
  
  public salvaAgregada({ component }: Button) {
    const form = Metadata.getInstance<Form>('formAgregada');
    const grid: GridEditable = Metadata.getInstance('AGREGADA');
    const editedRows = grid.getEditedRows();
    const central: GridEditable = Metadata.getInstance('CENTRAL');
    const currentRow = central.datasource.currentRow;

    if (!currentRow.CDFILIAL) this.utilities.showAlert('alertError', 'pme::centralNaoSelecionada', 'error');
    else if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.CDFILCENTPME = central.datasource.currentRow.CDFILIAL;
        row.IDTPFILIALCENT = central.datasource.currentRow.IDTPFILIALCENT;
      });
      
      if (grid) grid.datasource.loading = true;
      Http.post('/salvaAgregada', { editedRows }).then((response: IDictionary) => {
        if (response.data.length === 0) {
          grid.datasource.get();
          
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          this.gridEditableController.cancelEditing({ component });
        } else if (response.data.dataset.mensagem[0] === 'ERROR_UNID_CENTRAL') {
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_UNID_CENTRAL'), 'error');
        } else {
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_AGREGADA_M'), 'error');
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      }).finally(() => {
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  public confirmDeleteCentral({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiCentral',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteCentral() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteCentral() {
    const gridCentral = Metadata.getInstance<GridEditable>('CENTRAL');
    const checkedRows = gridCentral.selectedRows;
    Http.post('/excluiCentral', { requestType: 'Row', checkedRows }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        gridCentral.datasource.get();
        gridCentral.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
        DialogService.hide();
      } else {
        const mensagem = response.data.dataset.mensagem[0];
        this.utilities.showAlert('alertError', `pme::${mensagem}`, 'error');
        DialogService.hide();
      }
    }).catch((error) => {
      gridCentral.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    }).finally(() => {
      this.closeCardAgregada();
    });
  }
  
  public confirmDeleteAgregada({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiAgregada',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteAgregada() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteAgregada() {
    const gridAgregada = Metadata.getInstance<GridEditable>('AGREGADA');
    const checkedRows = gridAgregada.selectedRows;
    Http.post('/excluiAgregada', { requestType: 'Row', checkedRows }).then(() => {
      gridAgregada.datasource.get();
      gridAgregada.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
      DialogService.hide();
    }).catch((error) => {
      gridAgregada.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    });
  }

  public carregaAgregada({ component }: IEventParam<Datasource>) {
    const centralSelecionada = !!component.currentRow.CDFILIAL;

    if (centralSelecionada) {
      const agregada: Grid = Metadata.getInstance('AGREGADA');
      this.selectedCentral = `${component.currentRow.CDFILIAL} - ${component.currentRow.NMFILIAL}`;
    
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: component.currentRow.CDFILIAL,
        },
      ];
  
      agregada.datasource.filter = conditions;
      this.cardAgregadaVisible();
    } else {
      this.selectedCentral = `${component.currentRow.CDFILIAL} - ${component.currentRow.NMFILIAL}`;
      this.closeCardAgregada();
    }
  }
  
  private async createModalRelatorioCentral() {
    this.modalRelatorioCentral = await ModalService.createFromJson('planejamento/cadastro/modals/ger96000_relatorio_central', true);
  }
  
  public openModalRelatorioCentral() {
    if (!this.modalRelatorioCentral) {
      this.createModalRelatorioCentral().then(() => {
        this.modalRelatorioCentral.show();
      });
    } else {
      this.modalRelatorioCentral.show();
    }
  }
  
  public cancelModalRelatorioCentral() {
    this.modalRelatorioCentral.hide();
  }
  
  private setParams(row: IDictionary) {
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    const row: IDictionary = form.value;
    
    this.setParams(row);
    
    this.reportController.callGenerateReport(component, row, 'report/centrais');
  }
}
